#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  # This module generates content by an AI model
  # Can be used to generate titles, description, etc.
  module AI
    def self.generate(prompt)
      @config = YAML.load_file("#{Rails.root}/config/ai_models.yml")
      api_url = @config["api_url"]
      response = nil
      begin
        response = HTTParty.post(
          api_url,
          headers: { 'Content-Type' => 'application/json', 'Authorization' => 'Bearer no-key' },
          body: prompt.to_json,
          timeout: 300
        )
      rescue Net::ReadTimeout => e
        Rails.logger.error "Net::ReadTimeout while generating AI content.\nError: #{e.message}"
        return { error: 'HTTP request failed due to timeout', message: e.message }
      rescue StandardError => e
        return { error: 'HTTP request failed', message: e.message }
      end
      if response && response.success?
        response.parsed_response['choices'][0]['message']['content'].to_s.gsub(/"/, '').gsub(/<\/s>/, '').strip
      elsif response && response.code
        Rails.logger.error "HTTP request failed with response code: #{response.code}"
        { error: 'HTTP request failed', message: "Response code: #{response.code}" }
      else
        Rails.logger.error 'HTTP request failed without any response code'
        { error: 'HTTP request failed', message: 'Unknown error' }
      end
    end

    # Creates the prompt of an incident description
    def self.generate_incident_description(params)
      Rails.logger.info "Generating description of an incident...\n"
      alerts = params[:alerts]
      policies = ''
      alerts.each do |alert|
        policies += "#{alert[:msg]}\n"
      end

      @config = YAML.load_file("#{Rails.root}/config/ai_models.yml")

      prompt = {
        model: 'LLaMA_CPP',
        max_tokens: 1000,
        messages: [
          {
            role: 'system',
            content: "#{@config['generate_description_system_prompt']}"
          },
          {
            role: 'user',
            content: "#{@config['generate_description_user_prompt']}:\n#{policies}\n"
          }
        ]
      }
      generate(prompt)
    end

    # Creates a incident title
    def self.generate_incident_title(params)
      @config = YAML.load_file("#{Rails.root}/config/ai_models.yml")
      Rails.logger.info "Generating title of an incident...\n"
      alerts = params[:alerts]
      policies = ''
      alerts.each do |alert|
        policies += "#{alert[:msg]}\n"
      end

      prompt = {
        model: 'LLaMA_CPP',
        max_tokens: 255,
        messages: [
          {
            role: 'system',
            content: "#{@config['generate_title_system_prompt']}"
          },
          {
            role: 'user',
            content: "#{@config['generate_title_user_prompt']}:\n#{policies}\n"
          }
        ]
      }

      generate(prompt)
    end
  end
end
