#######################################################################
# Copyright (c) 2024 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  module Jobs
    # This jobs generates content: titles, description, etc. depending of the function
    class AIGenerateContentJob < Struct.new(:verbose, :user_id, :function, :params)
      include Redborder::Jobs::JobHelper

      def perform
        content = ('Node executing this job: ' + `hostname`.chomp! + "\n\n")
        begin
          response = nil
          content += "Generating #{function[:type]}...\n" if verbose
          klass = Object.const_get(function[:function_class])
          response = klass.send(function[:function_name], params)
          content += "Generated: '#{response}'\n"
        rescue StandardError => e
          content += "Failed to generate content.\nContent type: #{function[:type]}\n" if verbose
          content += "Error: #{e.message}\n"
        ensure
          content += "Creating StoredDelayedJob\n" if verbose
          StoredDelayedJob.create(
            job: 'AIGenerateContentJob',
            options: { humanize: "Generating: #{function[:type]}" },
            content: content,
            user_id: user_id
          )
        end
      end
    end
  end
end
